@@ -10,6 +10,7 @@
 
 #include <QDebug>
 #include <QDomDocument>
+#include <QHash>
 #include <QLocale>
 #include <QQueue>
 #include <QRegularExpression>
@@ -154,6 +155,21 @@ KCalcToken::TokenCode KCalcParser::stringToToken(const QString &buffer, int &ind
         }
     }
 
+    if (ANGLE_STR == s.first(1)) {
+        index++;
+        switch (trigonometric_Mode_) {
+        case RADIANS:
+            return KCalcToken::TokenCode::POLAR_RAD;
+        case GRADIANS:
+            return KCalcToken::TokenCode::POLAR_GRAD;
+        case DEGREES:
+            return KCalcToken::TokenCode::POLAR_DEG;
+        default:
+            Q_UNREACHABLE();
+            return KCalcToken::TokenCode::INVALID_TOKEN;
+        }
+    }
+
     if (s.startsWith(PLUS_STR)) {
         index++;
         return KCalcToken::TokenCode::PLUS;
@@ -194,6 +210,11 @@ KCalcToken::TokenCode KCalcParser::stringToToken(const QString &buffer, int &ind
         index++;
         return KCalcToken::TokenCode::PERMILLE;
     }
+    if (s.startsWith(I_STR) && !s.startsWith(IM_STR)) {
+        index++;
+        token_KNumber_ = I_STR;
+        return KCalcToken::TokenCode::KNUMBER;
+    }
     if (s.startsWith(DIVISION_STR) || s.startsWith(SLASH_STR) || s.startsWith(DIVISION_SLASH_STR)) {
         index++;
         return KCalcToken::TokenCode::DIVISION;
@@ -323,6 +344,10 @@ KCalcToken::TokenCode KCalcParser::stringToToken(const QString &buffer, int &ind
         index += 4;
         return KCalcToken::TokenCode::TANH;
     }
+    if (s.startsWith(CONJ_STR)) {
+        index += 4;
+        return KCalcToken::TokenCode::CONJ;
+    }
 
     if (s.startsWith(SIN_STR)) {
         index += 3;
@@ -373,6 +398,23 @@ KCalcToken::TokenCode KCalcParser::stringToToken(const QString &buffer, int &ind
         }
     }
 
+    if (s.startsWith(ARG_STR)) {
+        index += 3;
+        switch (trigonometric_Mode_) {
+        case RADIANS:
+            return KCalcToken::TokenCode::ARG_RAD;
+            break;
+        case GRADIANS:
+            return KCalcToken::TokenCode::ARG_GRAD;
+            break;
+        case DEGREES:
+            return KCalcToken::TokenCode::ARG_DEG;
+            break;
+        default:
+            break;
+        }
+    }
+
     if (s.startsWith(EXP_STR)) {
         index += 3;
         return KCalcToken::TokenCode::EXP;
@@ -434,6 +476,14 @@ KCalcToken::TokenCode KCalcParser::stringToToken(const QString &buffer, int &ind
         index += 2;
         return KCalcToken::TokenCode::LN;
     }
+    if (s.startsWith(RE_STR)) {
+        index += 2;
+        return KCalcToken::TokenCode::RE;
+    }
+    if (s.startsWith(IM_STR)) {
+        index += 2;
+        return KCalcToken::TokenCode::IM;
+    }
 
     if (index + 2 <= buffer.size()) {
         s = buffer.sliced(index, 2);
@@ -716,6 +766,9 @@ const QString KCalcParser::TokenToString(KCalcToken::TokenCode tokenCode)
     case KCalcToken::TokenCode::I:
         return I_STR;
         break;
+    case KCalcToken::TokenCode::POLAR:
+        return ANGLE_STR;
+        break;
     case KCalcToken::TokenCode::POS_INFINITY:
         return POS_INFINITY_STR;
         break;
@@ -815,15 +868,14 @@ const QString KCalcParser::TokenToString(KCalcToken::TokenCode tokenCode)
 int KCalcParser::loadConstants(const QDomDocument &doc)
 {
     QDomNode n = doc.documentElement().firstChild();
-    constant_ constant;
 
     while (!n.isNull()) {
         QDomElement e = n.toElement();
         if (!e.isNull() && e.tagName() == QLatin1String("constant")) {
-            constant.value = e.attributeNode(QStringLiteral("value")).value();
-            constant.symbol = e.attributeNode(QStringLiteral("symbol")).value();
+            const QString value = e.attributeNode(QStringLiteral("value")).value();
+            const QString symbol = e.attributeNode(QStringLiteral("symbol")).value();
 
-            constants_.append(constant);
+            constants_.insert(symbol, value);
         }
         n = n.nextSibling();
     }
@@ -838,13 +890,10 @@ int KCalcParser::loadConstants(const QDomDocument &doc)
 //------------------------------------------------------------------------------
 bool KCalcParser::constantSymbolToValue_(const QString &constantSymbol)
 {
-    QList<constant_>::const_iterator i;
-    for (i = constants_.constBegin(); i != constants_.constEnd(); ++i) {
-        if (i->symbol == constantSymbol) {
-            token_KNumber_ = i->value;
-            m_inputHasConstants = true;
-            return true;
-        }
+    if (constants_.contains(constantSymbol)) {
+        token_KNumber_ = constants_.value(constantSymbol);
+        m_inputHasConstants = true;
+        return true;
     }
 
     return false;
